/**
 * @file
 * Provides Splide extensions for onwheel event.
 */

(function ($, _ds, _win, _doc) {

  'use strict';

  var xZoom = function (Splide, Components) {
    var EL_SPLIDE_ROOT = Splide.root;
    var OPT = Splide.options;
    var OZ = OPT.zoom || {};
    var V_MAX = OZ.max || 1.5;
    var V_MIN = OZ.min || 1;
    var IS_ENABLED = OZ.on || false;
    var IS_SCALABLE = OZ.scale || false;
    var IS_CLICKABLE = OZ.click || false;
    var S_ZOOM_ROOT = OZ.root || false;
    var C_NICK = OZ.nickClass || 'sbox';
    var C_NICK_ZOOM = 'sz';
    var C_IS_NICK = 'is-' + C_NICK;
    var C_IS_SLIDE = 'is-slide';
    var C_ZOOM_ROOT = C_IS_NICK + '-zoomed';
    var C_ZOOM_XL = C_IS_NICK + '-zoom-xl';
    var S_ZOOM_PARENT = OZ.target || '.slide__media';
    var C_PINCH = C_IS_NICK + '-zoom';
    var C_DRAG = C_IS_NICK + '-dragging';
    var C_TARGET = C_IS_NICK + '-moved';
    var MIN_DEVICE_WIDTH = OZ.minWidth || '768px';
    var USE_TRANSFORM = OZ.transform || false;
    var IS_PINCHED = false;
    var C_IS_ZOOMABLE = 'is-zoomable';
    var S_IS_ZOOMABLE = '.' + C_IS_ZOOMABLE;
    var V_POS = {
      x: 0,
      y: 0
    };
    var EL_ZOOM_ROOT = null;
    var EL_ZOOM_PARENT = null;
    var EL_ZOOMED = null;
    var V_SIZES = {};
    var V_WIDTH;
    var V_HEIGHT;
    var EL_ZOOM_PARENTS = [];
    var V_WINSIZE = $.windowSize() || {};
    var V_TOGGLING;
    var V_LAST_SCALE = V_MAX;

    function dataset(el) {
      var set = el.dataset;
      var nw = $.toInt(set.sNw, 0);
      var nh = $.toInt(set.sNh, 0);
      var cw = $.toInt(set.sCw, 0);
      var ch = $.toInt(set.sCh, 0);
      var sm = nh < V_WINSIZE.height && nw < V_WINSIZE.width;

      return {
        nw: nw,
        nh: nh,
        cw: cw,
        ch: ch,
        ah: $.attr(el, 'height', nh),
        aw: $.attr(el, 'width', nw),
        fit: $.toInt(set.sFit, 0),
        sm: sm,
        lg: nw > cw && nh > ch,
        xl: nh >= V_WINSIZE.height
      };
    }

    function drag(v) {
      Splide.options = {
        drag: v
      };
    }

    // https://stackoverflow.com/questions/3971841
    // @todo remove for $.image.scale() post blazy:2.26.
    function aspectRatio(srcWidth, srcHeight, maxWidth, maxHeight) {
      var ratio = Math.min(maxWidth / srcWidth, maxHeight / srcHeight);

      return {
        width: srcWidth * ratio,
        height: srcHeight * ratio,
        ratio: ratio
      };
    }

    function removableClass(cn, v) {
      $.addClass(cn, C_IS_NICK + '-' + v);

      _win.clearTimeout(V_TOGGLING);

      V_TOGGLING = setTimeout(function () {
        V_TOGGLING = null;

        $.removeClass(cn, C_IS_NICK + '-' + v);
      }, 250);
    }

    function toTransform(x, y, scale) {
      var str = 'translate3d(' + x + 'px, ' + (y || 0) + 'px, 0)';

      if (!$.isUnd(scale)) {
        str += ' scale3d(' + scale + ', ' + scale + ', 1)';
      }

      return str;
    }

    function inject(el, x, y, scale, transform) {
      var tg = USE_TRANSFORM ? $.closest(el, S_ZOOM_PARENT) : el;

      if (!tg) {
        return;
      }

      if (USE_TRANSFORM) {
        tg = $.closest(el, S_ZOOM_PARENT);
        tg.style.transform = toTransform(x, y, scale);
      }
      else {
        tg.style.left = x === '' ? '' : x + 'px';
        tg.style.top = y === '' ? '' : y + 'px';
      }

      // $.on(tg, 'transitionend.' + C_NICK_ZOOM, function () {
      // tg.style.transition = '';
      // });
      // setTimeout(function () {
      // tg.style.transition = '';
      // }, 700);
    }

    function checkDim(el) {
      if (!el) {
        return {};
      }

      var set = dataset(el);
      var cw = set.cw;
      var ch = set.ch;
      var ah = set.ah;
      var aw = set.aw;
      var fit = set.fit;

      V_WIDTH = fit ? aw : cw;
      V_HEIGHT = fit ? ah : ch;

      return set;
    }

    function updateDim(el, fit) {
      if (!el || !el.dataset) {
        return;
      }

      var set = checkDim(el);
      var cw = V_WIDTH || set.cw;
      var ch = V_HEIGHT || set.ch;

      // el.style.width = cw + 'px';
      // el.style.height = (fit ? eh : ch) + 'px';

      var picture = $.closest(el, 'picture');
      if ($.isElm(picture)) {
        picture.style.width = cw + 'px';
        picture.style.height = ch + 'px';
      }

      var tg = $.closest(el, S_ZOOM_PARENT);
      if (tg) {
        // @todo remove for is-slide-xl.
        $[set.xl ? 'addClass' : 'removeClass'](tg, C_ZOOM_XL);

        var slide = $.closest(tg, '.slide');

        $.addClass(slide, C_IS_ZOOMABLE + '-slide');
      }
    }

    return {
      currSlide: null,
      canZoom: true,
      sizes: function (img) {
        V_SIZES = _ds.checkSizes(img || EL_ZOOMED, EL_ZOOM_PARENT);
        return V_SIZES;
      },

      isOffset: function (img, x, y) {
        var me = this;
        me.sizes(img);

        var nw = V_SIZES.nw / 3;
        var wh = V_WINSIZE.height / 3;
        var sy = Math.ceil((V_SIZES.h - V_SIZES.ph) / 3);

        return Math.abs(y) >= sy && Math.abs(y) > wh || Math.abs(x) > nw;
      },

      isZoomed: function () {
        return IS_ENABLED && $.hasClass(this.zoomRoot(), C_ZOOM_ROOT);
      },

      toggleClass: function (zooming, cls, delayed) {
        var el = this.zoomRoot() || EL_ZOOM_ROOT;
        if (el) {
          if (delayed) {
            if (zooming) {
              $.addClass(el, cls || C_ZOOM_ROOT);
            }
            else {
              setTimeout(function () {
                $.removeClass(el, cls || C_ZOOM_ROOT);
              }, delayed);
            }
          }
          else {
            $[zooming ? 'addClass' : 'removeClass'](el, cls || C_ZOOM_ROOT);
          }
        }
      },

      on: function () {
        this.canZoom = true;
      },

      off: function () {
        this.canZoom = false;
      },

      onMounted: function () {
        var me = this;

        var checkSizes = function () {
          V_WINSIZE = $.windowSize() || {};
          EL_ZOOM_PARENTS = $.findAll(EL_SPLIDE_ROOT, S_ZOOM_PARENT);

          if (EL_ZOOM_PARENTS.length) {
            $.each(EL_ZOOM_PARENTS, function (el) {
              var img = $.find(el, S_IS_ZOOMABLE);

              if ($.isElm(img)) {
                if ($.isDecoded(img)) {
                  me.fit(img);
                }
                else {
                  $.one(img, 'load.' + C_NICK_ZOOM, me.fit.bind(me));
                }
              }
            });
          }
        };

        checkSizes();
        $.on(_win, 'resize.' + C_NICK_ZOOM + ' orientationchange.' + C_NICK_ZOOM, $.debounce(checkSizes, 301));
      },

      onActive: function (slide) {
        var me = this;

        me.currSlide = slide;
        EL_ZOOMED = $.find(slide.slide, S_IS_ZOOMABLE);

        if (me.currSlide) {
          EL_ZOOM_PARENT = $.find(me.currSlide.slide, S_ZOOM_PARENT);
          if ($.isElm(EL_ZOOM_PARENT)) {
            EL_ZOOMED = $.find(EL_ZOOM_PARENT, S_IS_ZOOMABLE);

            // @todo remove for is-slide-fit.
            if (EL_ZOOMED && EL_ZOOMED.dataset) {
              var fit = $.toInt(EL_ZOOMED.dataset.sFit, 0) === 1;

              if (fit) {
                $.addClass(EL_ZOOM_PARENT, 'is-zoom-fit');
              }
            }
          }

          me.on();
        }
      },

      onWheel: function (e, data) {
        var me = this;
        var el = e.target;

        if ($.matchMedia(MIN_DEVICE_WIDTH, 'min')) {
          if (me.isZoomed()) {
            var cn = me.zoomRoot();

            $.addClass(cn, C_IS_NICK + '-wheeled');
            V_LAST_SCALE = data.scale;

            inject(el, data.x, data.y, V_LAST_SCALE || V_MAX);
          }
          else {
            me.canZoom = data.dir === 'down';
            me.zoomon(el, true);
          }
        }
      },

      mount: function () {
        var me = this;

        if (!IS_ENABLED) {
          return;
        }

        Splide.on('mounted.spz', function () {
          setTimeout(function () {
            me.onMounted();
          }, 300);
        });

        Splide.on('active.spz', me.onActive.bind(me));

        Splide.on('inactive.spz', function (slide) {
          var oldImg = $.find(slide.slide, S_IS_ZOOMABLE);

          // V_POS.x = 0;
          // V_POS.y = 0;
          if ($.isElm(oldImg)) {
            me.zoomOut(oldImg);
          }
        });

        // Must be outside splide events to support destroyed splide.
        me.dragon();
      },

      fit: function (el) {
        var me = this;
        var img = el.target || el;

        // if (!img.dataset) {
        // return;
        // }
        me.sizes(img);

        var ph = V_WINSIZE.height;
        var pw = V_WINSIZE.width;
        var ow = img.naturalWidth;
        var oh = img.naturalHeight;
        var eh = $.attr(img, 'height', oh);
        var nh = $.toInt(oh, 0);
        var nw = $.toInt(ow, 0);
        var xl = nh >= V_WINSIZE.height;
        var min;
        var cls;

        if ($.matchMedia('1700px', 'min')) {
          min = 210;
        }
        else if ($.matchMedia('1400px', 'min')) {
          min = 180;
        }
        else {
          min = 120;
        }

        var as = aspectRatio(ow, oh, pw - min, ph - min);
        var cw = $.toInt(as.width, 0);
        var ch = $.toInt(as.height, 0);
        var ratio = as.ratio;
        var fit = eh < ph;
        var slide = $.closest(img, '.slide');

        img.dataset.sFit = fit ? 1 : 0;
        img.dataset.sNw = nw;
        img.dataset.sNh = nh;
        img.dataset.sCw = cw;
        img.dataset.sCh = ch;
        img.dataset.sRatio = ratio;

        if (xl) {
          cls = 'xl';
        }
        else if (fit) {
          cls = 'fit';
        }
        else if (nh < ph && nw < pw) {
          cls = 'sm';
        }

        if (cls) {
          $.addClass(slide, C_IS_SLIDE + '-' + cls);
        }

        setTimeout(function () {
          updateDim(img, fit);
        });
      },

      zoomRoot: function () {
        if (!$.isElm(EL_ZOOM_ROOT) && S_ZOOM_ROOT) {
          EL_ZOOM_ROOT = typeof S_ZOOM_ROOT === 'string' ? $.find(_doc, S_ZOOM_ROOT) : S_ZOOM_ROOT;
        }
        return $.isElm(EL_ZOOM_ROOT) ? EL_ZOOM_ROOT : EL_SPLIDE_ROOT;
      },

      zoomIn: function (el) {
        var me = this;
        var valid = el && el.dataset;

        if (!valid) {
          return false;
        }

        var set = dataset(el);
        var nh = set.nh;
        var nw = set.nw;
        var fit = set.fit === 1;
        var small = set.sm;
        var largeEnough = set.lg;

        if (small || fit || !IS_SCALABLE) {
          me.toggleClass(false);

          if (small && largeEnough) {
            // @todo.
          }
          else {
            return false;
          }
        }

        me.toggleClass(true);

        var limit = V_WINSIZE.width > V_WINSIZE.height ? V_WINSIZE.width : V_WINSIZE.height;
        var as = aspectRatio(nw, nh, limit - 80, nh - 80);

        el.style.width = as.width + 'px';
        el.style.height = as.height + 'px';

        removableClass(me.zoomRoot(), 'zoomin');

        setTimeout(function () {
          inject(el, V_POS.x, V_POS.y, V_LAST_SCALE || V_MAX);
          me.off();
          drag(false);
        });
      },

      zoomOut: function (el) {
        var me = this;

        if (el) {
          var cn = $.closest(el, S_ZOOM_PARENT);
          // var set = checkDim(el);
          // var w = V_WIDTH || set.cw;

          if (USE_TRANSFORM) {
            cn.style.transform = 'scale3d(' + V_MIN + ', ' + V_MIN + ', ' + V_MIN + ')';
          }
          else {
            el.style.left = '';
            el.style.top = '';
          }

          var transend = function () {
            el.style.width = ''; // w + 'px';
            el.style.height = '';
            // el.style.transition = '';
            // cn.style.transition = '';
            // V_POS.x = 0;
            // V_POS.y = 0;
          };

          // if (me.isZoomed()) {
          // $.on(cn, 'transitionend.' + C_NICK_ZOOM, transend);
          // }
          setTimeout(transend, 650);

          removableClass(me.zoomRoot(), 'zoomout');

          me.toggleClass(false, C_ZOOM_ROOT, 100);
        }

        setTimeout(function () {
          me.on();
          drag(true);
        });
      },

      zoomon: function (el, reset) {
        var me = this;
        el = el || EL_ZOOMED;
        var img = $.hasClass(el, C_IS_ZOOMABLE) ? el : $.find(el, S_IS_ZOOMABLE);

        if (me.canZoom) {
          me.zoomIn(img);
        }
        else {
          me.zoomOut(img);
          if (reset) {
            inject(img, '', '', 1);
          }
        }
      },

      onClick: function (e, data) {
        var me = this;
        var img = e.target;

        if ($.hasClass(img, C_IS_ZOOMABLE)) {
          V_POS.x = data.x;
          V_POS.y = data.y;

          EL_ZOOMED = img;

          if (IS_PINCHED) {
            me.zoomOut(img);
            IS_PINCHED = false;
          }
          else {
            //  && !$.hasClass(me.zoomRoot(), C_DRAG)
            if (!_ds.doubletap()) {
              me.zoomon(img);
            }
          }
        }
      },

      onZoomIn: function (e, data) {
        var me = this;
        var img = e.target;

        if ($.hasClass(img, C_IS_ZOOMABLE)) {
          EL_ZOOMED = img;

          // me.zoomon(img);
          me.toggleClass(true);
        }
      },

      onZoomOut: function (e, data) {
        var me = this;
        var img = e.target;

        if ($.hasClass(img, C_IS_ZOOMABLE)) {
          EL_ZOOMED = img;

          me.zoomOut(img);
        }
      },

      onPinched: function (e, data) {
        IS_PINCHED = true;
      },

      dragon: function () {
        var me = this;
        var cn = me.zoomRoot();
        var opts = OZ;
        var el = EL_ZOOM_PARENT;
        var phase;
        // var dir;
        // var reset = false;
        // var x = 0;
        // var y = 0;

        function start(data) {
          V_POS.x = data.x;
          V_POS.y = data.y;

          el = el || data.el;
          var img = data.img;
          var tg = USE_TRANSFORM ? el : img;

          tg.style.transition = '';

          $.removeClass(me.zoomRoot(), C_IS_NICK + '-wheeled');
        }

        function move(data) {
          V_POS.x = data.x;
          V_POS.y = data.y;

          el = data.el || el;
          var img = data.img;
          var touchData = data.zoom;
          var inoutCls = C_PINCH +
            'in ' + C_PINCH + 'out';

          me.toggleClass(false, inoutCls);

          if (me.isZoomed()) {
            if (touchData) {
              me.toggleClass(true, C_PINCH + touchData);
            }
            else {
              me.toggleClass(false, inoutCls, 500);

              V_LAST_SCALE = data.scale === 1 ? V_LAST_SCALE : data.scale;

              // if ($.matchMedia(MIN_DEVICE_WIDTH, 'min')) {
              inject(img, V_POS.x, V_POS.y, V_LAST_SCALE);
              // }
            }
          }
          else {
            // me.zoomOut(img);
            me.toggleClass(false, inoutCls, 500);
          }
        }

        function end(data) {
          V_POS.x = data.x;
          V_POS.y = data.y;

          /*
          var img = data.img;
          el = data.el || el;
          dir = data.dir;
          var reset = me.isOffset(img, data.x, data.y);
          var downward = dir === 'down';
          // var upward = dir === 'up';
          var toRight = dir === 'right';
          var toLeft = dir === 'left';
          var horiz = toRight || toLeft;

          if (reset) {
            if (horiz) {
              x = toRight ? 0 : -1;
            }
            else {
              y = data.y > 0 ? sy : -sy;
            }
          }

          $[reset ? 'addClass' : 'removeClass'](me.zoomRoot(), C_IS_NICK + '-reset');

          setTimeout(function () {
            if (reset && data.trigger === data.event.type) {
              inject(img, x, y, V_LAST_SCALE || data.scale);
              V_POS.x = x;
              V_POS.y = y;
            }

            $.removeClass(me.zoomRoot(), C_IS_NICK + '-reset');
          }, 1200);
          */
        }

        var callback = function (e, data) {
          el = data.el || el;
          // dir = data.dir;
          phase = data.phase;

          data.img = $.find(el, S_IS_ZOOMABLE);
          me.sizes(data.img);

          if (phase === 'start') {
            start(data);
          }
          else if (phase === 'move') {
            move(data);
          }
          else if (phase === 'end') {
            end(data);
          }
        };

        opts.onClick = IS_CLICKABLE ? me.onClick.bind(me) : false;
        opts.onZoomIn = me.onZoomIn.bind(me);
        opts.onZoomOut = me.onZoomOut.bind(me);
        opts.onPinched = me.onPinched.bind(me);
        opts.onWheel = me.onWheel.bind(me);
        opts.callback = callback;
        opts.dragClass = C_DRAG;
        opts.activatedClass = C_ZOOM_ROOT;
        opts.zoomClass = C_IS_ZOOMABLE;
        opts.targetClass = C_TARGET;
        opts.max = V_MAX;

        var items = [];
        $.each(Components.Elements.slides, function (slide) {
          var item = $.find(slide, S_ZOOM_PARENT);

          if ($.isElm(item)) {
            items.push(item);
          }
        });

        if (items.length) {
          opts.elms = items;

          new SwipeDetect(cn, opts);
        }
      }
    };
  };

  _ds.extend({
    xZoom: xZoom
  });

})(dBlazy, dSplide, this, this.document);
